import os
import re
import argparse

def find_files(paths,ext):
    """
    Search for files with a specific extension in a list of directories.

    Args:
        dirs (list of str): A list of directories to search through.
        ext (str): The file extension to search for (e.g., ".txt", ".py").

    Returns:
        list of str: A list containing the full paths of files that match the given extension.
    """
    fileslist=[]
    for path in paths:
        if os.path.isdir(path):
            for root, _, files in os.walk(path, topdown=True):
                for file in files:
                    if file.endswith(ext):
                        fileslist.append(os.path.join(root, file))
        elif os.path.isfile(path):
            # If the path is a single file, check its extension and add if it matches
            if path.endswith(ext):
                fileslist.append(path)
    fileslist.sort()
    return fileslist

def extract_f90(dirs, pattern=None, label=None):
    """
    Extracts all indices names inside NEMO source code (.F90)

    Parameters:
    - directory: Path to the directory containing the Fortran files
    - pattern: pattern to look for the indices in the code

    Returns:
    The set of selected indices type inside NEMO
    """
    words = set()
    current_block = []
    inside_namelist = False

    files=find_files(dirs,".F90")

    for file in files:
        if label == 'modules':
            words.add(os.path.basename(file))
        else:
            with open(file, 'r', encoding='utf-8', errors='ignore') as fhandle:
                if label == 'namelist variables':  # NAMELIST block parsing for variables
                    for line in fhandle:
                        line = re.sub(r'!.*$', '', line).strip()  # Remove comments
                        # Check for start of NAMELIST block
                        if re.search(r"NAMELIST\s*/", line):
                            inside_namelist = True

                        # check variable within the namelist block
                        if inside_namelist:
                            current_block.append(line)
                            if not line.endswith('&'):  # End of NAMELIST block
                                block_content = " ".join(current_block).replace("&", " ")
                                block_content = re.sub(r'NAMELIST\s*/[^/]+/', '', block_content)
                                variables = re.findall(r'\b\w+\b', block_content)
                                words.update([variable.lower() for variable in variables])
                                current_block = []
                                inside_namelist = False
                else:
                    content = fhandle.read()
                    matches = pattern.findall(content)
                    for match in matches:
                        match = match.lower()
                        if label == 'cpp keys':
                            words.add(f"key_{match}")
                        elif label == 'namelist blocs':
                            words.add(f"nam{match}")
                        else :
                            words.add(match)
    return words

def extract_latex(dirs, pattern=None, label=None):
    """
    Extracts all indices (of a same type) referenced in LaTeX documentation using the format.

    Parameters:
    - dirs: List of path to the directory containing the LaTeX files
    - regex: pattern format to look for the indices in the documentation

    Returns:
    The set of selected indices type inside Latex
    """
    words = set()

    files=find_files(dirs,'.tex')

    for file in files:
        with open(file, 'r', encoding='utf-8', errors='ignore') as fhandle:
            # Read the file content
            content = fhandle.read()
            # Find all matches for the pattern
            matches = {match.replace('\\_', '_').lower() for match in pattern.findall(content)}
            for match in matches:
                match=match.lower()
                if label == 'namelist blocs' :
                    words.add(f"nam{match}")
                elif label == 'modules':
                    words.add(f"{match}.F90")
                elif label == 'cpp keys':
                    words.add(f"key_{match}")
                else :
                    words.add(match)
    return words

def print_differences(argmiss_doc, argmiss_f90, label="namelist blocs"):
    """
    Prints a comparison of namelist blocks that are either:
            - missing from documentation
            - missing from the F90 code

    Parameters:
    - missing1 (list of str): items defined in F90 but not documented.
    - missing2 (list of str): items documented but not defined in F90.
    - label (str): A descriptive label for the output text (default is "namelist blocks").

    This function outputs:
    - A list of items that are defined in the F90 code but are not documented.
    - A list of items that are documented but not defined in the F90 code.

    If there are no missing or obsolete items, 
    it prints a message stating that all blocks are accounted for.
    """

    # Check for namelist blocks defined in F90 but not documented
    if argmiss_doc:
        print(f"\nThe following {label} are not listed in the LaTeX documentation:")
        for argmiss in sorted(argmiss_doc):
            print(f"- {argmiss}")
    elif argmiss_doc == set():
        print(f"\nAll {label} are documented in the LaTeX files.")

    # Check for namelist blocks documented but not in F90
    if argmiss_f90:
        print(f"\nThe following {label} are listed in the Latex documentation but not present in the F90 code:")
        for argmiss in sorted(argmiss_f90):
            print(f"- {argmiss}")
    elif argmiss_f90 == set():
        print(f"\nAll {label} that are documented in the LaTeX files are in the F90.")

def check_indices(f90_directory, latex_directory, argcheck, argcheckdoc, argcheckf90):
    """
    Checks if all the indices of one type are listed in the LaTeX documentation
    & vice versa.
    Parameters:
    - f90_directory: The path to the Fortran source code directory.
    - latex_directory: The path to the LaTeX documentation directory.
    - argcheck: name of the indices to check ( namelist bloc, subroutine, modules, namelist parameters, cpp key)
    """
    # Choose the regex pattern based on argcheck
    if argcheck == 'cpp':
        latex_pattern = re.compile(r"\\key\{([A-Za-z0-9-_]+(?:\\_[A-Za-z0-9]+)*)\}")
        f90_pattern = re.compile(r"#(?:ifdef|ifndef|if|elif)\s+(?:!?\s*defined\s*)?\(?[kK]ey_([A-Za-z0-9_]+)\)?")
        clabel = 'cpp keys'
    elif argcheck == 'mod':
        latex_pattern = re.compile(r"\\mdl\{([A-Za-z0-9_]+)\}")
        f90_pattern = None
        clabel = 'modules'
    elif argcheck == 'nambloc':
        latex_pattern = re.compile(r"\\nam\{([A-Za-z0-9_]+)\}")
        f90_pattern = re.compile(r"NAMELIST\s*/\s*nam([A-Za-z0-9_]+)\s*/")
        clabel = 'namelist blocs'
    elif argcheck == 'namvar':
        latex_pattern = re.compile(r"\\np(?:\[=[^]]*\])?\{([A-Za-z0-9_]+)\}")
        f90_pattern = re.compile(r"NAMELIST\s*/\s*nam([A-Za-z0-9_]+)\s*/")
        clabel = 'namelist variables'
    else:
        raise ValueError("Invalid check type. Use cpp, mod, sub, nambloc or namvar")

    f90_out = extract_f90(f90_directory,f90_pattern,clabel)
    latex_out = extract_latex(latex_directory,latex_pattern,clabel)

    # Find in F90 but not documented
    missing_out1=None
    if argcheckdoc:
        missing_out1 = f90_out - latex_out

    # Documented but not defined in F90
    missing_out2=None
    if argcheckf90:
        missing_out2 = latex_out - f90_out

    # print differences between F90 and latex
    print_differences(missing_out1, missing_out2, clabel)

    return missing_out1, missing_out2

def main(f90_directory, latex_directory, checkargs, noF90, nodoc):
    """
    Main function to check for missing or extra indices and print the results.

    Arguments:
    - f90_directory: Path to the Fortran source code directory
    - latex_directory: Path to the LaTeX documentation directory
    - checkarg: name of the indices to check ( namelist bloc, subroutine, modules, namelist parameters, cpp key)
    """

    for argcheck in checkargs:
        print('')
        print(f'=================================================================')
        print(f'                     Check {argcheck}                            ')
        print(f'=================================================================')
        _ , _ = check_indices(f90_directory, latex_directory, argcheck, not noF90, not nodoc)

if __name__ == "__main__":
    # Set up argument parser
    parser = argparse.ArgumentParser(description="Compare NEMO source code and LaTeX documentation.")
    parser.add_argument("-F90",
        type=str,
        nargs='+',
        required=True,
        help="Path to the directory containing Fortran source code")

    parser.add_argument("-latex",
        type=str,
        nargs='+',
        help="Path to the directory containing LaTeX documentation")

    parser.add_argument("--check",
        type=str,
        choices=["namvar", "nambloc", "mod", "cpp"],
        nargs='+',
        default=["namvar", "nambloc", "mod", "cpp"],
        required=False,
        help="Indices to check (choose one or more from: namvar, nambloc, mod, cpp, sub)"
    )

    parser.add_argument("--noF90",action="store_true", 
                        help="Only check indices from the doc (ie obsolete parameters)")
    
    parser.add_argument("--nodoc",action="store_true", 
                        help="Only check indices from the F90 (ie new parameters)")

    # Parse arguments
    args = parser.parse_args()

    main(args.F90, args.latex, args.check, args.noF90, args.nodoc)
