import os
import re
import unittest
from unittest.mock import patch
from io import StringIO
from check_indices_not_documented import extract_f90, extract_latex, print_differences, check_indices, find_files

class TestNamelistChecker(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        """
        Set up a temporary directory structure with sample Fortran and LaTeX files for testing.
        This method runs once for the entire class.
        """
        os.makedirs('test_dir/src', exist_ok=True)
        os.makedirs('test_dir/latex', exist_ok=True)

        # Create a sample Fortran file with namelist blocks
        with open('test_dir/src/test1.F90', 'w') as f:
            f.write("test to detect cpp key, namelist bloc and namelist variables\n")
            f.write("NAMELIST /namtest1/ ln_test1,         nn_TEST1_a,  & \n")
            f.write("   &                                  nn_test1_b     \n")
            f.write("#ifdef key_TEST_1\n")
            f.write("#ifndef key_test_1\n")

        with open('test_dir/src/test2.F90', 'w') as f:
            f.write("test to detect cpp key, namelist bloc and namelist variables\n")
            f.write("#if  defined   key_test2\n")
            f.write("#if defined key_test2\n")
            f.write("NAMELIST /namtest2/ ln_test2 \n")

        with open('test_dir/src/test3.F90', 'w') as f:
            f.write("test to detect cpp key test3\n")
            f.write("#ifndef   key_test3\n")

        # Create a sample LaTeX file referencing namelist blocks
        with open('test_dir/latex/doc1.tex', 'w') as f:
            f.write(r"test to detect key \key{test\_1} \n")
            f.write(r"test to detect namelist parameter in \np[=.true.]{ln_test1}{ln\_test1}) and \np[=0..3]{nn_test1_a}{nn\_test1_a})\n")
            f.write(r"                                                                        or  \np[=0..3]{nn_test1_b}{nn\_test1_b}\n")
            
        with open('test_dir/latex/doc2.tex', 'w') as f:
            f.write(r"test to detect module : \mdl{test2}                       \n")
            f.write(r"test to detect namelist bloc name \nam{test2}             \n")
            f.write(r"test to detect namelist variable \np{ln_test2}{ln\_test2} \n")
        
        with open('test_dir/latex/doc3.tex', 'w') as f:
            f.write(r"test to detect namelist bloc name \nam{test3} and variable \np[=.true.]{ln_test3}{ln\_test3})\n")
            f.write(r"test to detect \mdl{test3} and keys \key{test4} and \key{test3}.\n")

    @classmethod
    def tearDownClass(cls):
        """
        Clean up the temporary directory structure after all tests have run.
        This method runs once for the entire class.
        """
        import shutil
        shutil.rmtree('test_dir')

# test find file
    def test_find_files(self):
        """Test find file."""
        out = find_files(['test_dir'], '.F90')
        expected_out = ['test_dir/src/test1.F90', 'test_dir/src/test2.F90', 'test_dir/src/test3.F90']
        self.assertEqual(out, expected_out)

# test nambloc
    def test_extract_f90_nambloc(self):
        """Test extracting namelist bloc from Fortran."""
        regex = re.compile(r"NAMELIST\s*/\s*nam([A-Za-z0-9_]+)\s*/") #re.compile(r'NAMELIST\s*/\s*([A-Za-z0-9_]+)\s*/')
        out = extract_f90(['test_dir/src'], regex, 'namelist blocs')
        expected_out = {'namtest1', 'namtest2'}
        self.assertEqual(out, expected_out)

    def test_extract_latex_nambloc(self):
        """Test extracting namelist bloc from Doc."""
        regex = re.compile(r"\\nam\{([A-Za-z0-9_]+)\}")
        out = extract_latex(['test_dir/latex'], regex, 'namelist blocs')
        expected_out = {'namtest2','namtest3'}
        self.assertEqual(out, expected_out)

# test namvar
    def test_extract_f90_namvar(self):
        """Test extracting namelist variable from Fortran."""
        regex = re.compile(r"NAMELIST\s*/\s*nam([A-Za-z0-9_]+)\s*/")
        out = extract_f90(['test_dir/src'], regex, 'namelist variables')
        expected_out = {'ln_test1', 'nn_test1_a', 'nn_test1_b', 'ln_test2'}
        self.assertEqual(out, expected_out)

    def test_extract_latex_namvar(self):
        """Test extracting namelist variable from Doc."""
        regex = re.compile(r"\\np(?:\[=[^]]*\])?\{([A-Za-z0-9_]+)\}")
        out = extract_latex(['test_dir/latex'], regex, 'namelist variables')
        expected_out = {'ln_test1', 'nn_test1_a', 'nn_test1_b', 'ln_test2', 'ln_test3'}
        self.assertEqual(out, expected_out)

# test module
    def test_extract_f90_modules(self):
        """Test extracting modules from Fortran."""
        regex = None
        out = extract_f90(['test_dir/src'], regex, 'modules')
        expected_out = {'test1.F90','test2.F90','test3.F90'}
        self.assertEqual(out, expected_out)

    def test_extract_latex_modules(self):
        """Test extracting modules from Doc."""
        regex = re.compile(r"\\mdl\{([A-Za-z0-9_]+)\}")
        out = extract_latex(['test_dir/latex'], regex, 'modules')
        expected_out = {'test2.F90', 'test3.F90'}
        self.assertEqual(out, expected_out)

# test cpp
    def test_extract_f90_cppkeys(self):
        """Test extracting modules from Fortran."""
        regex = re.compile(r"#(?:ifdef|ifndef|if|elif)\s+(?:defined\s*)?\(?[kK]ey_([A-Za-z0-9_]+)\)?")
        out = extract_f90(['test_dir/src'], regex, 'cpp keys')
        expected_out = {'key_test_1','key_test2','key_test3'}
        self.assertEqual(out, expected_out)

    def test_extract_latex_cppkeys(self):
        """Test extracting modules from Doc."""
        regex = re.compile(r"\\key\{([A-Za-z0-9-_]+(?:\\_[A-Za-z0-9]+)*)\}")
        out = extract_latex(['test_dir/latex'], regex, 'cpp keys')
        expected_out = {'key_test_1','key_test3', 'key_test4'}
        self.assertEqual(out, expected_out)

# test differences
    # missmatch
    @patch('sys.stdout', new_callable=StringIO)
    def test_print_differences_nomatch(self, mock_stdout):
        """Test the output of print_differences function."""

        missing1 = {'namtest1','namtest2'}
        missing2 = {'namtest3'}

        clabel='namelist blocs'
        print_differences(missing1, missing2, clabel)

        # Capture the printed output
        output = mock_stdout.getvalue()

        # Define the expected output
        expected_output = (
            f"\nThe following {clabel} are not listed in the LaTeX documentation:\n"
            f"- namtest1\n"
            f"- namtest2\n"
            f"\nThe following {clabel} are listed in the Latex documentation but not present in the F90 code:\n"
            f"- namtest3\n"
        )

        self.assertEqual(output, expected_output)

    # match
    @patch('sys.stdout', new_callable=StringIO)
    def test_print_differences_match(self, mock_stdout):
        """Test the output of print_differences function."""

        missing1 = set()
        missing2 = set()

        clabel='namelist blocs'
        print_differences(missing1, missing2, clabel)

        # Capture the printed output
        output = mock_stdout.getvalue()

        # Define the expected output with exact line breaks
        expected_output = (
            f"\nAll {clabel} are documented in the LaTeX files.\n"
            f"\nAll {clabel} that are documented in the LaTeX files are in the F90.\n"
        )

        self.assertEqual(output, expected_output)

# test check indices
# nambloc
    @patch('sys.stdout', new_callable=StringIO)
    def test_check_indices_nambloc(self, mock_stdout):
        """Test the output of check indices function for nambloc."""

        clabel='namelist blocs'
        _,_=check_indices(['test_dir/src'], ['test_dir/latex'], 'nambloc', True, True)

        # Capture the printed output
        output = mock_stdout.getvalue()

        # Define the expected output
        expected_output = (
            f"\nThe following {clabel} are not listed in the LaTeX documentation:\n"
            f"- namtest1\n"
            f"\nThe following {clabel} are listed in the Latex documentation but not present in the F90 code:\n"
            f"- namtest3\n"
        )

        self.assertEqual(output, expected_output)
# namvar
# cppkeys
# modules

if __name__ == '__main__':
    unittest.main()
